// SPDX-License-Identifier: (GPL-2.0-only OR BSD-3-Clause)
/*
 * Copyright (c) 2023 MediaTek Inc.
 */

#include <command.h>
#include <common.h>
#include <malloc.h>
#include <stdio.h>
#include <debug_impl.h>
#include <MsTypes.h>
//#include <ShareType.h>
////#include <MsStr.h>

#include <dmalloc.h>
#include <CommonDataType.h>
#include <demura.h>
#include <demuraDll.h>
#include <demura_config.h>
#include <ms_utils.h>
#include <vendor/decoder_input.h>


MS_BOOL init_demura_heap(void)
{
 #if 0
    MS_U32 base_addr = 0;
    MS_U32 buf_len   = CONVERT_BUFFER_SIZE;

    if (get_large_memory(&base_addr, buf_len) == FALSE)
    {
        UBOOT_ERROR("failed to get Demura_Convert_Buf\n");
        return FALSE;
    }

    // init dmalloc buffer
    dmalloc_init((unsigned char *)base_addr, buf_len);
#endif
    return TRUE;
}

MS_BOOL Alloc_LutIn_Space(void *pdat_info, BinOutputInfo *pbin_info)
{
    int n_Hnode     = pbin_info->HNode;
    int n_Vnode     = pbin_info->VNode;
    int nTotalNode  = n_Hnode * n_Vnode;
    int level_count = pbin_info->LevelCount;

    if ((n_Hnode == 0) || (n_Vnode == 0) || (level_count == 0))
    {
        UBOOT_ERROR("n_Hnode = %d, n_Vnode = %d, level_count = %d\n", n_Hnode, n_Vnode, level_count);
        UBOOT_ERROR("Alloc_LutIn_Space Failed !\n");
        return FALSE;
    }

    #if defined (CONFIG_DEMURA_URSA11)
        int n_Layer     = 6;     // Wait to fix it.
        int layer_size  = sizeof(double) * nTotalNode;
        interface_info *pChanData = (interface_info *)pdat_info;

        int idx, channel;
        for(channel = 0; channel < 3; channel++)
        {
            double **lut_in  = (double **)malloc(sizeof(double *) * 6);

            for(idx = 0; idx < n_Layer; idx++)
            {
                char *pLut_in = dmalloc(layer_size);
                CHECK_DMALLOC_SPACE(pLut_in, layer_size);
                lut_in[idx] = (double *)pLut_in;
            }
            pChanData[channel].Lut_in = lut_in;
        }

    #elif defined (CONFIG_DEMURA_URSA13)
        int n_Layer     = level_count > 0 ? level_count : 8;
        int layer_size  = sizeof(strgb_structInfo) * nTotalNode;
        interface_info *pDataInfo = (interface_info *)pdat_info;

        int idx;
        for (idx = 0; idx < n_Layer; idx++)
        {
            char *pLut_in  = (char *)dmalloc(layer_size);
            CHECK_DMALLOC_SPACE(pLut_in, layer_size);
            pDataInfo->Lut_in[idx] = (strgb_structInfo *)pLut_in;
        }
        for (idx = n_Layer; idx < 8; idx++)
        {
            pDataInfo->Lut_in[idx] = NULL;   // Init to NULL value
        }
    #elif (defined (CONFIG_DEMURA_M7622) || defined (CONFIG_DEMURA_M7632))   // Morton case
        int n_Layer     = level_count > 0 ? level_count : 8;
        int layer_size  = sizeof(strgb_structInfo) * nTotalNode;
        interface_info *pDataInfo = (interface_info *)pdat_info;

        int idx;
        for (idx = 0; idx < n_Layer; idx++)
        {
            char *pLut_in  = (char *)dmalloc(layer_size);
            CHECK_DMALLOC_SPACE(pLut_in, layer_size);
            pDataInfo->Lut_in[idx] = (strgb_structInfo *)pLut_in;
        }
        for (idx = n_Layer; idx < 8; idx++)
        {
            pDataInfo->Lut_in[idx] = NULL;   // Init to NULL value
        }
    #elif defined (CONFIG_DEMURA_MT5896)   // M6
        int n_Layer     = level_count > 0 ? level_count : 9;
        int layer_size  = sizeof(strgb_structInfo) * nTotalNode;
        interface_info *pDataInfo = (interface_info *)pdat_info;

        int idx;
        for (idx = 0; idx < n_Layer; idx++)
        {
            char *pLut_in  = (char *)dmalloc(layer_size);
            CHECK_DMALLOC_SPACE(pLut_in, layer_size);
            pDataInfo->Lut_in[idx] = (strgb_structInfo *)pLut_in;
        }
        for (idx = n_Layer; idx < 8; idx++)
        {
            pDataInfo->Lut_in[idx] = NULL;   // Init to NULL value
        }
    #endif

    return TRUE;
}

#if defined(CONFIG_DEMURA_VENDOR_MULTI)
int do_demura_convert(BinOutputInfo *pbin_info, EN_DEMURA_MULTI_VENDOR multi_vendor)
{
    MS_BOOL ret; // fix coverity
    //char  cmd[256];

    if (check_str_resume() == 1)
    {
        //UBOOT_DEBUG("Can not covert demura file during STR\n");
        return 0;
    }

    UBOOT_DEBUG("=================================\n");
    UBOOT_DEBUG("DEMURA_URSA_TYPE : %s\n", CONFIG_DEMURA_URSA_STRING);
    UBOOT_DEBUG("DEMURA_VENDOR    : %s\n",  CONFIG_DEMURA_VENDOR_STRING);
    UBOOT_DEBUG("=================================\n");

    if (init_spi_flash() != TRUE)
    {
        UBOOT_ERROR("init_spi_flash error!\n");
        return -1;
    }

//================================ Generate Demura Bin ===============================

    #if defined (CONFIG_DEMURA_URSA11)
        interface_info DataInfo[3];
    #elif defined (CONFIG_DEMURA_URSA13)
        interface_info DataInfo;
    #elif (defined (CONFIG_DEMURA_M7622) || defined (CONFIG_DEMURA_M7632) || defined (CONFIG_DEMURA_MT5896))
        interface_info DataInfo;
    #else
        #error "Unkown DEMURA_URSA_TYPE !"
    #endif

    memset(&DataInfo, 0, sizeof(DataInfo));

    if(multi_vendor == EN_DEMURA_MULTI_AUO)
        ret = Decode_To_Mstar_Format_AUO((void *)&DataInfo, pbin_info);
    else if((multi_vendor > EN_DEMURA_MULTI_LGD_START) && (multi_vendor < EN_DEMURA_MULTI_LGD_END))
        ret = Decode_To_Mstar_Format_LGD((void *)&DataInfo, pbin_info, multi_vendor);
    else if(multi_vendor == EN_DEMURA_MULTI_NOVA)
        ret = Decode_To_Mstar_Format_NOVA((void *)&DataInfo, pbin_info, multi_vendor, NOVA_Shift);
    else if(multi_vendor == EN_DEMURA_MULTI_CSOT_HISILICON)
        ret = Decode_To_Mstar_Format_CSOT_HISILICON((void *)&DataInfo, pbin_info);
    else if(multi_vendor == EN_DEMURA_MULTI_CSOT_HIMAX)
        ret = Decode_To_Mstar_Format_CSOT_HIMAX((void *)&DataInfo, pbin_info);
    else if(multi_vendor == EN_DEMURA_MULTI_SDC)
        ret = Decode_To_Mstar_Format_SDC((void *)&DataInfo, pbin_info);
    else if(multi_vendor == EN_DEMURA_MULTI_INX)
        ret = Decode_To_Mstar_Format_INX((void *)&DataInfo, pbin_info);
    else if(multi_vendor == EN_DEMURA_MULTI_CSOT_CSOT)
        ret = Decode_To_Mstar_Format_CSOT_CSOT((void *)&DataInfo, pbin_info);
    else if(multi_vendor == EN_DEMURA_MULTI_HKC_NOVA)
        ret = Decode_To_Mstar_Format_NOVA((void *)&DataInfo, pbin_info, multi_vendor, HKC_NOVA_Shift); //HKC's demura use NOVA format, just offset 0x1000
    else if(multi_vendor == EN_DEMURA_MULTI_BOE_ESWIN)
        ret = Decode_To_Mstar_Format_BOE_ESWIN((void *)&DataInfo, pbin_info);
    else if (multi_vendor == EN_DEMURA_MULTI_H_K_C_NOVA_120HZ)
    {
        ret = Decode_To_Mstar_Format_NOVA((void *)&DataInfo, pbin_info, multi_vendor, H_K_C_NOVA_Shift_120HZ); //H_K_C's demura use NOVA format, just offset 0x2000
    }
    else
        ret = FALSE;

    if (ret == FALSE)
    {
        UBOOT_ERROR("Decode_To_Mstar_Format error\n");
        return -1;
    }

    if (mstar_demura_interface((interface_info *)&DataInfo, pbin_info) == FALSE)
    {
        UBOOT_ERROR("demura interface error\n");
        return -1;
    }

//================================ End Generate Demura Bin ===============================

    MS_U8  *pout = pbin_info->bin_buf;
    MS_U32 fsize = pbin_info->bin_size;

    printf("output_addr    = 0x%p\n", pout);
    printf("bin_size       = 0x%x\n", (uint)fsize);
    printf("dmalloc_length = 0x%x\n", get_dmalloc_length());

    // Enable SPI_FLASH - Write protect
    /*memset(cmd, 0, sizeof(cmd));
    snprintf(cmd, sizeof(cmd), "sf protect lock");
    if (run_command(cmd, 0) != 0)
    {
      printf("Enable SPI_FLASH Write protect Fail !\n");
    }
    memset(cmd, 0, sizeof(cmd));
    snprintf(cmd, sizeof(cmd), "sf protect status");
    run_command(cmd, 0);
    */
    printf("\n");
    return 0;
}
#else
int do_demura_convert(BinOutputInfo *pbin_info)
{
    MS_BOOL ret = TRUE;
    //char  cmd[256];

    if (check_str_resume() == 1)
    {
        //UBOOT_DEBUG("Can not covert demura file during STR\n");
        return 0;
    }

    UBOOT_DEBUG("==================================\n");
    UBOOT_DEBUG("DEMURA_URSA_TYPE : U%s\n", CONFIG_DEMURA_URSA_STRING);
    UBOOT_DEBUG("DEMURA_VENDOR    : %s\n",  CONFIG_DEMURA_VENDOR_STRING);
    UBOOT_DEBUG("==================================\n");

    if (init_spi_flash() != TRUE)
    {
        UBOOT_ERROR("init_spi_flash error!\n");
        return -1;
    }

//================================ Generate Demura Bin ===============================

    #if defined (CONFIG_DEMURA_URSA11)
        interface_info DataInfo[3];
    #elif defined (CONFIG_DEMURA_URSA13)
        interface_info DataInfo;
    #elif (defined (CONFIG_DEMURA_M7622) || defined (CONFIG_DEMURA_M7632))
        interface_info DataInfo;
    #else
        #error "Unkown DEMURA_URSA_TYPE !"
    #endif

    memset(&DataInfo, 0, sizeof(DataInfo));

    // Ready to standard covert flow, Now comment .
    // Alloc_LutIn_Space(pdat_info, pbin_info)

    ret = Decode_To_Mstar_Format((void *)&DataInfo, pbin_info);
    if (ret == FALSE)
    {
        UBOOT_ERROR("Decode_To_Mstar_Format error\n");
        return -1;
    }

    mstar_demura_interface((interface_info *)&DataInfo, pbin_info);

//================================ End Generate Demura Bin ===============================

    MS_U8  *pout = pbin_info->bin_buf;
    MS_U32 fsize = pbin_info->bin_size;

    printf("output_addr    = 0x%p\n", pout);
    printf("bin_size       = 0x%x\n", (uint)fsize);
    printf("dmalloc_length = 0x%x\n", get_dmalloc_length());

    // Enable SPI_FLASH - Write protect
    /*memset(cmd, 0, sizeof(cmd));
    snprintf(cmd, sizeof(cmd), "sf protect lock");
    if (run_command(cmd, 0) != 0)
    {
      printf("Enable SPI_FLASH Write protect Fail !\n");
    }
    memset(cmd, 0, sizeof(cmd));
    snprintf(cmd, sizeof(cmd), "sf protect status");
    run_command(cmd, 0);
    */
    printf("\n");
    return 0;
}
#endif


void set_demura_version(int version)
{
    demura_set_version(version);
}

void set_demura_pnl(MS_U16 width, MS_U16 height)
{
    demura_set_panelinfo(width, height);
}
